// Verkaufsautomat
`timescale 1 ns / 1 ps
module iglu_multicoin (
  input wire       CLOCK, // Takt
                   RESET, // Reset
  input wire [2:0] COIN,  // eingeworfene Muenze
  output reg       ICE);  // Warenausgabe

  `include "iglu_multicoin_defs.v"

  // interne Variable
  reg       PRESENT,      // jetziger Zustand
            NEXT;         // naechster Zustand

  reg [5:0] SUM;          // eingezahlter Geldbetrag
                          // Einheit 10 Cent
  reg [5:0] NEXT_SUM;     // neue Summe
                          // Einheit 10 Cent

  always @(PRESENT, COIN, SUM)      // <-- flankenfrei!
    case (PRESENT)
      INSERT_COINS: begin           // Muenzeinwurf
        case (COIN)
          X0:                       // kein Einwurf
            NEXT_SUM = SUM + 0;
          X10:                      // 10 Cent
            NEXT_SUM = SUM + 1;
          X20:                      // 20 Cent
            NEXT_SUM = SUM + 2;
          X50:                      // 50 Cent
            NEXT_SUM = SUM + 5;
          X100:                     // 1 Euro
            NEXT_SUM = SUM + 10;
          X200:                     // 2 Euro
            NEXT_SUM = SUM + 20;
          default:                  // Don't care, Latch vermeiden
            NEXT_SUM = 'bx;
        endcase
        if (SUM >= Price)           // Ueberzahlung wird ignoriert
          {NEXT,ICE} = {ICECREAM, No};
        else
          {NEXT,ICE} = {INSERT_COINS, No};
      end
      ICECREAM: begin               // genug bezahlt
        NEXT_SUM = 0;               // eingezahlter Betrag wieder auf 0
        {NEXT,ICE} =                // Warenausgabe
          {INSERT_COINS, Yes};
      end
    endcase

  // naechsten Zustand mit jedem Takt endgueltig zum jetzigen machen;
  // synchrones Reset
  //
  always @(posedge CLOCK)             // <-- flankengesteuert!
    if (RESET == 1'b1) begin
      PRESENT <= INSERT_COINS;        // Anfangszustand
      SUM <= 0;
    end
    else begin
      PRESENT <= NEXT;
      SUM <= NEXT_SUM;
    end

endmodule // iglu_multicoin
